// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include "Matrix.H"
#include "Cholesky.H"
#include "ConvexQP.H"
using namespace std;

int main()
{
  
  Matrix<double> G(2,2), Ci(2,3), Ce;
  Vector<double> x(2), ci(3), ce;

  // function is 1/2 x^T G x + b^T x
  // initially store b in x
  G[0][0] = G[1][1] =  4;
  G[0][1] = G[1][0] = -2;
  x[0] = 6;
  x[1] = 0;

  // constraints
  Ci[0][0] = 1; Ci[1][0] = 0; ci[0] = 0;
  Ci[0][1] = 0; Ci[1][1] = 1; ci[1] = 0;
  Ci[0][2] = 1; Ci[1][2] = 1; ci[2] = -2;

  // convert G to L
  Cholesky( G );

  // solve for unconstrained optimium
  solveLLH( G, x );
  for( int i=0; i<2; i++ ) x[i] = -x[i];

  // impact of constraints
  double df = 0;
  int code = ConvexQP( G, x, Ce, ce, Ci, ci, df, 1.e-16 );
  cout << setprecision(14);
  cout << "code=" << code << " Convex Quadratic Solution x=" << endl << x << endl;
  cout << "change to residual " << df << endl;
}
