// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <math.h>
#include "Matrix.H"
#include <assert.h>
#include "FFT.H"
using namespace std;

int main()
{
  cout << scientific << setprecision(4) << showpos;

  Vector<double> fR(8);
  Vector<double> fI(8);

  fR[0] = 1; fR[1] = 2; fR[2] = 3; fR[3] = 0;
  fR[4] = -1; fR[5] =-3; fR[6] =-2; fR[7] =-1;

  fI[0] = 0; fI[1] = 0; fI[2] = 0; fI[3] = 0;
  fI[4] = 0; fI[5] = 0; fI[6] = 0; fI[7] = 0;

  cout << "start"  << endl;
  cout << "fR" << fR;
  cout << "fI" << fI;

  fft( fR, fI, true );

  cout << "forward"  << endl;
  cout << "fR" << fR;
  cout << "fI" << fI;

  // report as coefficients A_ and B_ of cos/sin functions
  int N = fR.size(); // assumed even
  assert( N%2 == 0 );

  int M=N/2;
  cout << "A0 " << 2*fR[0] << 2*fI[0] << "i" << endl;

  for( int j=1; j<M; j++ ) {
    cout << "A" << noshowpos << j << showpos << " " <<
    fR[j] + fR[N-j] << fI[j] + fI[N-j] << "i" << endl;
    cout << "B" << noshowpos << j << showpos << " " <<
    fI[N-j] - fI[j] << fR[j] - fR[N-j] << "i" << endl;
  }

  cout << "A" <<  noshowpos << M << showpos << " "
       << 2*fR[M] << 2*fI[M] << "i" << endl;
  

  fft( fR, fI, false );

  cout << "backward"  << endl;
  cout << "fR" << fR;
  cout << "fI" << fI;
}
