// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <math.h>
#include "Matrix.H"
#include "VMM.H"
using namespace std;

double func( const Vector<double>& x)
{
  // the Fletcher-Powell Helix
  double theta = atan(x[1]/x[0])/(2*M_PI) + ((x[0]<0)?0.5:0);
  double r = sqrt( x[0]*x[0] + x[1]*x[1] );
  double f = 100*( (x[2]-10*theta)*(x[2]-10*theta) + (r-1)*(r-1) ) + x[2]*x[2];
  return f;
}

void grad( const Vector<double>& x, Vector<double>& g )
{
  // grad of Fletcher-Powell Helix
  double r = sqrt(x[0]*x[0] + x[1]*x[1]);
  double drdx0 = x[0]/r;
  double drdx1 = x[1]/r;
  double theta = atan(x[1]/x[0])/(2*M_PI) + ((x[0]<0)?.5:0);
  double dtdx0 = -x[1]/(2*M_PI*r*r);
  double dtdx1 =  x[0]/(2*M_PI*r*r);
  g[0] = 200*(r-1)*drdx0 + 2000*(10*theta-x[2])*dtdx0;
  g[1] = 200*(r-1)*drdx1 + 2000*(10*theta-x[2])*dtdx1;
  g[2] = 202*x[2] - 2000*theta;
}

int main()
{
  Vector<double> x0(3);
  int iteration;
  cout << scientific << setprecision(4);

  x0[0]=-1; x0[1]=0; x0[2]=0;  VMM( func, grad, x0, DFP,  iteration, 1.e-9, 1.e-9 );
  cout << "DFP:   iterations=" << iteration << " solution=" << endl << x0 << endl;

  x0[0]=-1; x0[1]=0; x0[2]=0;  VMM( func, grad, x0, BFGS, iteration, 1.e-9, 1.e-9 );
  cout << "BFGS:  iterations=" << iteration << " solution=" << endl << x0 << endl;

  x0[0]=-1; x0[1]=0; x0[2]=0;  VMM( func, grad, x0, OS,   iteration, 1.e-9, 1.e-9 );
  cout << "OS:    iterations=" << iteration << " solution=" << endl << x0 << endl;
}
