// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <math.h>
#include "Matrix.H"
#include "GE.H"
using namespace std;

int main()
{
  int code, n = 3;
  Matrix<double> A(n,n);
  Vector<double> b(n);
  Vector<int>    pl(n), pr(n);

  for( int k=0; k<5; k++ ) {

    A[0][0] = 0.03;   A[0][1] = 300.;    A[0][2] = 8e7;
    A[1][0] = -0.995; A[1][1] = -140000; A[1][2] = 1.6e7;
    A[2][0] = 0.01;   A[2][1] = -19000;  A[2][2] = 1e6;

    b[0] = A[0][0]+A[0][1]+A[0][2];
    b[1] = A[1][0]+A[1][1]+A[1][2];
    b[2] = A[2][0]+A[2][1]+A[2][2];

    cout << "A" << endl << A << endl;
    cout << "b" << endl << b << endl;


    switch(k) {
    case 0: cout << "trivial pivoting" << endl;
            code = ge( A,pl,pr, TRIVIAL);
            break;
    case 1: cout << "partial pivoting" << endl;
            code = ge( A,pl,pr, PARTIAL);
            break;
    case 2: cout << "implicit scaled pivoting" << endl;
            code = ge( A,pl,pr, IMPLICITSCALEDPP);
            break;
    case 3: cout << "rook pivoting" << endl;
            code = ge( A,pl,pr, ROOK );
            break;
    case 4: cout << "complete pivoting" << endl;
            code = ge( A,pl,pr, COMPLETE );
            break;
    }
    if( code != 0 ) {
      cout << "error" << endl;
    } else {
      printLRP(A,pl,pr);
      solveLR( A, pl, pr, b );
      cout << "x= " << scientific  << setprecision(16) << endl << b << endl;
    }
  }
}
