// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#ifndef __Givens__H_
#define __Givens__H_
#include <math.h>

// find the coefficients of the Givens reflection matrix
// to eliminate y2:
// (c  s)(y1) = (*)
// (s -c)(y2)   (0)

inline void Givens( const double& y1, const double& y2,
                    double& c, double& s, double& v )
{
  double mu = fabs(y1);
  double tmp = fabs(y2);
  if( tmp > mu ) mu = tmp;

  if( mu == 0 ) {
    c = 1;
    s = 0;
    v = 0;
    return;
  }

  double s1 = y1/mu;
  double s2 = y2/mu;
  double p = mu*sqrt(s1*s1 + s2*s2);
  if( y1 < 0 ) p = -p;
  c = y1/p;
  s = y2/p;
  v = s/(1+c);
}
#endif
