// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <math.h>
#include "Matrix.H"

// a<b<c is an ordered set of abscissas, and fa=f(a), etc.
// if b is a + (gold-1)*(c-a), or
//         c - (gold-1)*(c-a),
// then self-similarity will be maintained.
// Stop when c-a <= eps.

void goldensection( const Vector<double>&x0, const Vector<double>&p,
                    double& a,  double& b,  double& c,
                    double& fa, double& fb, double& fc,
                    double eps, double (*funcp)(const Vector<double>&) )
{
  double d, fd, gold = (1.+sqrt(5.))/2.;
  int N = x0.size();
  Vector<double> x(N);

  // second test stops when c-a cannot be resolved given machine precision:
  // machine precision ~ ((sqrt(5)-1)/2)^76
  int count = 0;
  while( (c-a > eps) && (count++ <= 76) ) {

    // insert "d" in order a,b,d,c
    if( c-b > b-a ) {
      d = b + (2.-gold)*(c-b);
    } else {
      double tmp = b - (2.-gold)*(b-a);
      d = b;
      b = tmp;
    }
    for( int i=0; i<N; i++ ) x[i] = x0[i] + d*p[i];
    fd = (*funcp)( x );

    if( fb < fd ) {
      c = d; fc = fd;
    } else {
      a = b; fa = fb;
      b = d; fb = fd;
    }
  }
}
