// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <math.h>
#include "Matrix.H"
#include "GE.H"
#include "Cholesky.H"
#include "Householder.H"
using namespace std;

void makeHilbert( int n, Matrix<double>& A, Vector<double>& b )
{
  for( int i=0; i<n; i++ ) {
    b[i] = 0;
    for( int j=0; j<n; j++ ) {
      double t = 1./(i+j+1.);
      b[i] += t;
      A[i][j] = t;
    }
  }
}

int main()
{
  Matrix<double> A(20,20);
  Vector<double> b(20), diag(20);
  Vector<int>    pl(20), pr(20);
  int code;

  string getypes[5]={"trivial","partial","implicit scaled","rook","complete"};
  string qrtypes[4]={"trivial","Golub","Powell-Reid","Bjorck"};

  for( int n=5; n<=20; n+=5 ) {
    cout << "n=" << n << endl;
    A.resize(n,n);
    b.resize(n);
    diag.resize(n);
    pl.resize(n);
    pl.resize(n);

    for( int type=0; type<5; type++ ) {
      makeHilbert(n, A, b);
      code = ge( A, pl, pr, (GEPivotType)type );
      cout << "  GE with " << getypes[type] << " pivoting ("
           << (code? "fail" : "ok") << ")" << endl;
      if( !code ) {
        solveLR(A, pl, pr, b );
        double norm = 0;
        for( int i=0; i<n; i++ ) norm += (b[i]-1.)*(b[i]-1.);
        norm = sqrt(norm);
        cout << "    norm = " << norm << endl;
      }
    }

    for( int type=0; type<4; type++ ) {
      makeHilbert(n, A, b);
      code = Householder( A, pl, pr, diag, (QRPivotType)type );
      cout << "  Householder with " << qrtypes[type] << " pivoting ("
           << (code? "fail" : "ok") << ")" << endl;
      if( !code ) {
        solveQR( A, pl, pr, diag, b );
        double norm = 0;
        for( int i=0; i<n; i++ ) norm += (b[i]-1.)*(b[i]-1.);
        norm = sqrt(norm);
        cout << "    norm = " << norm << endl;
      }
    }

    makeHilbert(n, A, b);
    code = Cholesky(A);
    cout << "  Cholesky (" << (code? "fail" : "ok") << ")" << endl;
    if( !code ) {
      solveLLH( A, b );
      double norm = 0;
      for( int i=0; i<n; i++ ) norm += (b[i]-1.)*(b[i]-1.);
      norm = sqrt(norm);
      cout << "    norm = " << norm << endl;
    }
  }
}
