// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <stdlib.h>
#include <math.h>
#include "Matrix.H"
#include "Householder.H"
using namespace std;

int main()
{
  cout << scientific << setprecision(16) << showpos;

  Matrix<double> A(3,3);
  Vector<double> b(3);
  Vector<double> diag(3);
  Vector<int> Pl(3), Pr(3);
  double d = 1.e-10;
  A[0][0]=   2; A[0][1]=  -1; A[0][2]=   1;  b[0]=  2*(1.+d);
  A[1][0]=  -1; A[1][1]=   d; A[1][2]=   d;  b[1]=   -d;
  A[2][0]=   1; A[2][1]=   d; A[2][2]=   d;  b[2]=    d;

  int ret = Householder( A, Pl, Pr, diag, NOTHING );
  if( ret != 0 ) {
    cout << "Householder failed" << endl;
    exit(0);
  }

  printQR( A, Pl, Pr, diag );

  for( int i=0; i<A.columns(); i++ ) {
    cout << " column " << noshowpos << " k= " << showpos << diag[i] << endl;
    for( int j=i; j<A.rows(); j++ ) {
      cout << "  w" << noshowpos << j << " " << showpos << A[j][i] << endl;
    }
  }
  cout << "solve QRx = b for x:" << endl;
  solveQR( A, Pl, Pr, diag, b );
  for( int i=0; i<A.columns(); i++ )
    cout << "x" << noshowpos << i << " " << showpos << b[i] << endl;
}
