// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <math.h>
#include "Matrix.H"
#include "Householder.H"
#include "SVD.H"
using namespace std;

#define MAX(A,B) (((A)>(B))?(A):(B))

int main()
{
  cout << scientific << setprecision(4) << showpos;

  Matrix<double>A(6,5);
  Vector<double> b(6),  diag(5);
  Vector<int> Pl(6), Pr(5);
  double mu = 1.e-9;

  for( int i=0; i<6; i++ ) {
    for( int j=0; j<5; j++ ) A[i][j] = ((i==0) ? 1 : 0);
    if( i>0 ) A[i][i-1] = mu;
  }
  b[0]=mu; b[1]=b[5]=0; b[2]=b[4]=-5; b[3]=5;

  Householder( A, Pl, Pr, diag, POWELLREID );
  solveQR( A, Pl, Pr, diag, b );

  cout << "Householder:" << endl;
  for( int i=0; i<A.columns(); i++ )
    cout << "x" << noshowpos << i << " " << showpos << b[i] << endl;

  // rebuild A,b for SVD solve
  for( int i=0; i<6; i++ ) {
    for( int j=0; j<5; j++ ) A[i][j] = ((i==0) ? 1 : 0);
    if( i>0 ) A[i][i-1] = mu;
  }
  b[0]=mu; b[1]=b[5]=0; b[2]=b[4]=-5; b[3]=5;

  Matrix<double> U(6,6);
  Matrix<double> VT(5,5);
  Vector<double> sigma(5);
  SVD( A, U, VT, sigma );

  double smax = sigma[0];
  for( int i=1; i<A.columns(); i++ ) smax = MAX(smax,sigma[i]);

  // U sigma VT x = b
  // x = V 1/sigma UT b
  Vector<double> UTb(6);
  for( int i=0; i<6; i++ ) {
    UTb[i] = 0;
    for( int j=0; j<6; j++ ) UTb[i] += U[j][i]*b[j];
  }
  // divide by sigma where sigma not zero
  for( int i=0; i<5; i++ ) UTb[i] *= ((sigma[i]>=1e-10*smax) ? (1/sigma[i]) : 0);
  // and multiply by V, store result in b
  for( int i=0; i<5; i++ ) {
    b[i] = 0;
    for( int j=0; j<5; j++ ) b[i] += VT[j][i]*UTb[j];
  }

  cout << "SVD:" << endl;
  for( int i=0; i<A.columns(); i++ )
    cout << "x" << noshowpos << i << " " << showpos << b[i] << endl;
}
