// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <sstream>
#include "Matrix.H"
#include "Cholesky.H"
#include "ConvexQP.H"
using namespace std;

int main()
{
  stringstream ss( stringstream::in | stringstream::out );
  Matrix<double> G(5,5), Ci, Ce(5,3);
  Vector<double> x(5), ci, ce(3);
  cout << setprecision(14);

  ss << "4 4 0 0 0  4 22 2 0 0  0 2 2 0 0  0 0 0 2 0  0 0 0 0 2";
  ss >> G;
  
  ss.clear();
  ss << "1 0 0  3 0 1  0 1 0  0 1 0  0 -2 -1 0 0 0";
  ss >> Ce;
  ss >> ce;

  ss.clear();
  ss << "0 0 2 1 1";
  ss >> x;

  // convert G to L
  Cholesky( G );

  // impact of constraints
  double df = 0;
  int code = ConvexQP( G, x, Ce, ce, Ci, ci, df, 1.e-16 );
  cout << "code=" << code << " Convex Quadratic Solution x=" << endl << x << endl;
  cout << "change to residual " << df << endl;

  cout << "function " << (x[0]-x[1])*(x[0]-x[1]) + (x[1]+x[2]-2)*(x[1]+x[2]-2) +
                         (x[3]-1)*(x[3]-1) + (x[4]-1)*(x[4]-1) << endl;
  cout << "constraint1 " << x[0] + 3*x[1] << endl;
  cout << "constraint2 " << x[2] + x[3] - 2*x[4] << endl;
  cout << "constraint3 " << x[1] - x[4] << endl;
}
