// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
using namespace std;

// Note:  this is probably machine dependent.
// It "works" with IEEE-754 (1985 & 2008) with the "Round to ties to even"
// rounding mode.

template <class T> void precision()
{
  int t = 1;
  T   test, half = 0.5,  epsilon = 0.5;

  do {
    test = half + epsilon;
    if( test == half ) {

      // this can only happen if round(half+epsilon) = half,
      // which means epsilon is 2^{-(t+2)}:
      //
      //   if epsilon = 2^{-t}, half+epsilon would be machine
      //   representable.
      //
      //   if epsilon = 2^{-(t+1)}, then half+epsilon is not
      //   machine representable, and will not round up to
      //   something other than half in round-to-nearest mode.

      //   if epsilon 2^{-(t+1)}  and half+epsilon rounds up,
      //   then change the following two lines to
      //   t-=2; and epsilon *=4;

      t --;
      epsilon *= 2;

      // done

      cout << "machine epsilon is " << epsilon
           << " (2^" << -t << ")"
           << " with " << sizeof(T) << " byte precision" << endl;
      return;
    }

    // machine precision not yet found:  make epsilon smaller ...
    epsilon /= 2;
    t ++;

  } while(1);
}

int main(int argc, char **argv)
{
  precision<double>();
  precision<float>();
}
