// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#ifndef __RCP_H__
#define __RCP_H__
// reference counted pointer class for objects of type T
// created via "new T[]", i.e, requiring "delete []" vs "delete"
// for deallocation.

template <class T> class RCP
{
  public:

    inline RCP(T* ptr = 0)
    {
      m_ptr = ptr;
      m_count = NULL;
      if( m_ptr ) {
        m_count = new int;
        *m_count = 1;
      }
    }

    inline RCP( const RCP<T> &a_rcp )
    {
      m_ptr   = a_rcp.m_ptr;
      m_count = a_rcp.m_count;
      ++(*m_count);
    }

    inline ~RCP()
    {
      if( m_count != NULL && --(*m_count) == 0 ) {
        delete[] m_ptr;
        delete m_count;
        m_ptr   = NULL;
        m_count = NULL;
      }
    }

    inline const RCP<T>& operator =(const RCP<T>& a_rcp)
    {
      if( m_ptr != a_rcp.m_ptr ) {
        if( m_count != 0 && --(*m_count) == 0 ) {
          delete[] m_ptr;
          delete m_count;
        }
        m_ptr = a_rcp.m_ptr;
        m_count = a_rcp.m_count;
        if( m_count != NULL ) ++(*m_count);
      }
      return *this;
    }

    inline T& operator[] (int n)
    {
      return m_ptr[n];
    }

    const inline T& operator[] (int n) const
    {
      return m_ptr[n];
    }

  protected:
    T*   m_ptr;
    int* m_count;

};
#endif
