// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <sstream>
#include <iomanip>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "Matrix.H"
#include "Simplex.H"
using namespace std;

int main( int argc, char ** argv )
{
  cout << scientific << setprecision(4) << showpos;
  int n_var = 6;
  int n_eqn = 4;

  // worst case allocation for phase 1
  int mx_var = n_var + n_eqn;
  int mx_eqn = n_eqn + 1;

  // allocate for worst case
  Matrix<double> A(mx_eqn,mx_var);
  Vector<double> b(mx_eqn), x(mx_eqn);
  Vector<int> J_list(mx_eqn), K_list(mx_var-n_eqn), is_restricted(mx_var);

  // resize for standard form problem  (resizing down does not free
  // memory -- so if phase1() needs to resize up, it'll be "free"
  A.resize(n_eqn,n_var);
  b.resize(n_eqn);
  x.resize(n_eqn);
  J_list.resize(n_eqn);
  K_list.resize(n_var-n_eqn);
  is_restricted.resize(n_var);

  // data for a given problem
  stringstream ss( stringstream::in | stringstream::out );
  ss << "0.06 0.04 0.08 -1 0 0 1 1 1 0 0 0 \
         0 0 1 0 1 0 -1 4 0 0 0 1     0 10000 2000 0";
  ss >> A >> b;
  ss.clear();
  ss << "1 1 1 0 1 1";
  ss >> is_restricted;

  int t_var = 3; // objective variable
  int t_eqn = 0; // objective equation

  cout << "at start A " << endl << A << endl;
  cout << "at start b " << endl << b << endl;

  int retval = phase1( A, b, x, J_list, K_list, is_restricted, t_var, t_eqn );

  cout << "phase1 return value " << noshowpos << retval << showpos<< endl;
  cout << "after phase1 A " << endl << A << endl;
  cout << "after phase1 b " << endl << b << endl;
  cout << "after phase1 x " << endl << x << endl;
  cout << "after phase1 J " << endl << noshowpos << J_list << endl;
  cout << "after phase1 K " << endl << K_list << endl;
  cout << "after phase1 isrestricted " << endl << is_restricted << showpos << endl;

  if( retval != 0 ) return retval;

  retval = phase2( A, b, x, J_list, K_list, is_restricted, t_var );
  
  cout << "phase2 return value " << noshowpos << retval << showpos << endl;
  cout << "after phase2 x " << endl << x << endl;
  cout << "after phase2 J " << endl << noshowpos << J_list << endl;
  cout << "after phase2 K " << endl << K_list << showpos << endl;

  return retval;
}
