// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <math.h>
#include "Matrix.H"
#include "VMM.H"
using namespace std;

double func( const Vector<double>& x)
{
  // Rosenbrock's banana
  return 100.*(x[1]-x[0]*x[0])*(x[1]-x[0]*x[0]) + (1.-x[0])*(1.-x[0]);
}

void grad( const Vector<double>& x, Vector<double>& g )
{
  // Rosenbrock's banana
  g[0] = 2.*( -1. + x[0]*(1. - 200.*x[1] + 200.*x[0]*x[0]));
  g[1] = 200.*(x[1]-x[0]*x[0]);
}

int main()
{
  Vector<double> x0(2);
  int iteration;
  x0[0] = -1.2; x0[1]=1; VMM( func, grad, x0, DFP,  iteration, 1.e-9, 1.e-9 );
  cout << scientific << setprecision(4)
  << "DFP:   iterations=" << iteration << " solution=" << endl << x0 << endl;

  x0[0] = -1.2; x0[1]=1; VMM( func, grad, x0, BFGS, iteration, 1.e-9, 1.e-9 );
  cout << "BFGS:  iterations=" << iteration << " solution=" << endl << x0 << endl;

  x0[0] = -1.2; x0[1]=1; VMM( func, grad, x0, OS,   iteration, 1.e-9, 1.e-9 );
  cout << "OS:    iterations=" << iteration << " solution=" << endl << x0 << endl;
}

