// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <math.h>
#include "Matrix.H"
#include "GE.H"
#include "Householder.H"
using namespace std;

void makeVandermonde( int n, Matrix<double>& A, Vector<double>& b )
{
  for( int i=0; i<n; i++ ) {
    b[i] = 0;
    double xi = (n-1.-i)/(n-1.);
    double xij = 1;
    for( int j=0; j<n; j++ ) {
      b[i] += xij;
      A[i][j] = xij;
      xij *= xi;
    }
  }
}

int main()
{
  cout << scientific << setprecision(2);
  Matrix<double> A(16,16);
  Vector<double> b(16), diag(16);
  Vector<int>    pl(16), pr(16);
  int code;

  string getypes[5]={"trivial","partial","implicit scaled","rook","complete"};
  string qrtypes[4]={"trivial","Golub","Powell-Reid","Bjorck"};

  for( int n=4; n<=16; n+=4 ) {
    cout << "n=" << n << endl;
    A.resize(n,n);
    b.resize(n);
    diag.resize(n);
    pl.resize(n);
    pl.resize(n);

    for( int type=0; type<5; type++ ) {
      makeVandermonde(n, A, b);
      code = ge( A, pl, pr, (GEPivotType)type );
      cout << "  GE with " << getypes[type] << " pivoting ("
           << (code? "fail" : "ok") << ")" << endl;
      if( !code ) {
        solveLR(A, pl, pr, b );
        double norm = 0;
        for( int i=0; i<n; i++ ) norm += (b[i]-1.)*(b[i]-1.);
        norm = sqrt(norm);
        cout << "    norm = " << norm << endl;
      }
    }

    for( int type=0; type<4; type++ ) {
      makeVandermonde(n, A, b);
      code = Householder( A, pl, pr, diag, (QRPivotType)type );
      cout << "  Householder with " << qrtypes[type] << " pivoting ("
           << (code? "fail" : "ok") << ")" << endl;
      if( !code ) {
        solveQR( A, pl, pr, diag, b );
        double norm = 0;
        for( int i=0; i<n; i++ ) norm += (b[i]-1.)*(b[i]-1.);
        norm = sqrt(norm);
        cout << "    norm = " << norm << endl;
      }
    }
  }
}
