// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <fstream>
#include "Matrix.H"
#include "FFT.H"
#include <math.h>
using namespace std;

#define WINDOW

int main()
{
  int N     = 4096;
  int halfw = 10;

  Vector<double> rR(N), rI(N), cR(N), cI(N), fR(N), fI(N);

  // input stock data, save rate/price,  make a copy, create filter
  fstream data;
  data.open("dji", fstream::in );
  double price;
  data >> price;
  for( int i=0; i<N; i++ ) {
    double newprice;
    data >> newprice;
    rR[i] = (newprice-price)/price;
#ifdef WINDOW
    double w = 1. - fabs((2*i)/N-1.); // Bartlet window
    rR[i] *= w;
#endif
    cR[i] = rR[i];
    rI[i] = cI[i] = fI[i] = fR[i] = 0;
    if( i <= halfw || N-i <= halfw ) {
      fR[i] = 1./(2.*halfw+1);
    }
  }
  data.close();

  // smooth data:  convolution
  fft( cR, cI, true );
  fft( fR, fI, true );
  for( int i=0; i<N; i++ ) {
    double r1 = cR[i], r2 = fR[i];
    double i1 = cI[i], i2 = fI[i];
    cR[i] = N*( r1*r2 - i1*i2 );
    cI[i] = N*( r1*i2 + i1*r2 );
  }
  fft( cR, cI, false );

  // obtain noise by subtracting smoothed data
  double sum = 0;
  for( int i=0; i<N; i++ ) {
    rR[i] -= cR[i];
    sum += rR[i];
  }

  // find autocorrelation
  fft( rR, rI, true );
  for( int i=0; i<N; i++ ) {
    rR[i] = N*( rR[i]*rR[i] + rI[i]*rI[i] );
    rI[i] = 0;
  }
  // and complete autocorrelation
  fft( rR, rI, false );

  // variance is proportional to
  double variance = rR[0];

  // write in weird order for plotting
  fstream result;
  result.open( "autocorr.out", fstream::out );
  result << scientific << setprecision(15);
  for( int i=0; i<N; i++ ) {
    int j = i+N/2;
    j %= N;
   result << ( (j>=N/2) ? (j-N) : j ) << " " << rR[j]/variance << endl;
  }
  result.close();
}
