// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <fstream>
#include "Matrix.H"
#include "FFT.H"
#include <math.h>
using namespace std;

int main()
{
  int N=1024;
  double sigma= 10;
  double a1=10,  c1=100, w1=500;
  double a2=50,  c2=400, w2=15;
  Vector<double> fR(N), fI(N), gR(N), gI(N), convR(N), convI(N);

  // create F and G data, write
  fstream before;
  before.open("conv.before", fstream::out );
  before << scientific  << setprecision(16);
  for( int i=0; i<N; i++ ) {
    fR[i] = fI[i] = gR[i] = gI[i] = 0;
    // fake stectrum
    if( i < N/2 ) {
      fR[i]  = 0.25*w1*w1*a1/((i-c1)*(i-c1)+0.25*w1*w1) +
               0.25*w2*w2*a2/((i-c2)*(i-c2)+0.25*w2*w2);
    }
    // fake filter
    if( i <= 2*sigma )
       gR[i] = exp(-i*i/(2*sigma*sigma))/(sigma*sqrt(2.*M_PI));
    if( N-i <= 2*sigma )
       gR[i] = exp(-(N-i)*(N-i)/(2*sigma*sigma))/(sigma*sqrt(2.*M_PI));

    before << i << " " << fR[i] << " " << gR[i] << endl;
  }
  before.close();

  // FFT convolution:
  //   (1) make transform
  fft( fR, fI, true );
  fft( gR, gI, true );
  //   (2) multiply, multiply by N
  for( int i=0; i<N; i++ ) {
    convR[i] = N*( fR[i]*gR[i] - fI[i]*gI[i] );
    convI[i] = N*( fR[i]*gI[i] + fI[i]*gR[i] );
  }
  //   (3) back transform
  fft( convR, convI, false );

  // write
  fstream after;
  after.open("conv.after", fstream::out );
  after << scientific  << setprecision(16);
  for( int i=0; i<N; i++ ) {
    after << i << " " << convR[i] << " " << convI[i] << endl;
  }
  after.close();
}
