// G. Miller, Numerical Analysis for Engineers and Scientists,
// Cambridge University Press, Cambridge, 2014.
// http://www.cambridge.org/9781107021082

#include <iostream>
#include <iomanip>
#include <math.h>
using namespace std;

double TSR_bad(int N)
{
  double x = 0,  h = 1./N,  I = 0;
  for( int i=0; i<=N; i++ ) {
    double f = (i==0) ? 0 : expm1(x)/sqrt(x);
    I += f*h*( i%N ? 1 : 0.5 );
    x += h;
  }
  return I;
}

double TSR_good(int N)
{
  double u = 0,  h = 1./N,  I = 0;
  for( int i=0; i<=N; i++ ) {
    double f = 2*expm1(u*u);
    I += f*h*( i%N ? 1 : 0.5 );
    u += h;
  }
  return I;
}

int main()
{
  cout << scientific << setprecision(8);

  cout << "bad" << endl;
  int N = 100;
  double I0=0, I1=0, I2=0;
  for( int i=0; i<6; i++ ) {
    I2=I1; I1=I0; I0=TSR_bad(N);
    cout << 1./N << " " << I0;
    if( i >= 2 )
      cout << " " << log(fabs(I2-I1)/fabs(I1-I0))/log(2.);
    cout << endl;
    N *= 2;
  }

  cout << "good" << endl;
  N = 100;
  I0=0, I1=0, I2=0;
  for( int i=0; i<6; i++ ) {
    I2=I1; I1=I0; I0=TSR_good(N);
    cout << 1./N << " " << I0;
    if( i >= 2 )
      cout << " " << log(fabs(I2-I1)/fabs(I1-I0))/log(2.);
    cout << endl;
    N *= 2;
  }

  return 0;
}

